# encoding: utf-8
require "logstash/inputs/base"
require "logstash/namespace"
require "stud/interval"
require "socket" # for Socket.gethostname
require 'rest-client'

# Generate a repeating message.
#
# This plugin is intented only as an example.

class LogStash::Inputs::Attunity_Metrics < LogStash::Inputs::Base
  config_name "attunity_metrics"

  default :codec, "json"

  config: url, :validate => :string

  config: username, :validate => :string

  config: password, :validate => :string

  config: verify_ssl, :validate => :bool, :default => true

  public
  def register
    
    @header = {
      :content_type => "application/json",
      :user => @username,
      :password => @password
    }

    @client = resource = RestClient::Resource.new(
      @url,
      :verify_ssl => OpenSSL::SSL::VERIFY_NONE
    )

    @logger.info("Registering attunity_metrics", :url => @url, :user => @username, :password => @password, :verify_ssl => @verify_ssl)
  end

  def run(queue)
    # we can abort the loop if stop? becomes true
    # while !stop?
    #   event = LogStash::Event.new("message" => @message, "host" => @host)
    #   decorate(event)
    #   queue << event
    #   # because the sleep interval can be big, when shutdown happens
    #   # we want to be able to abort the sleep
    #   # Stud.stoppable_sleep will frequently evaluate the given block
    #   # and abort the sleep(@interval) if the return value is true
    #   Stud.stoppable_sleep(@interval) { stop? }
    # end # loop

    login()

    servers = get_servers()
    servers.each do |server|
      server_details = get_server_details(server.name)
      tasks = get_tasks(server.name)

      tasks.each do |task|
        task_detail = get_task_details(server.name, task.name)

        event = LogStash::Event.new("message" => @message, "host" => @host)
        decorate(event)
        queue << event
      end
    end

    logout()

  end # def run

  def stop
    # nothing to do in this case so it is not necessary to define stop
    # examples of common "stop" tasks:
    #  * close sockets (unblocking blocking reads/accepts)
    #  * cleanup temporary files
    #  * terminate spawned threads
  end

  private
  def login()
    response = @client["api/v1/login"].get(
      :header => @header
    )
    
    @header[:'EnterpriseManager.APISessionID'] = response.header[:'EnterpriseManager.APISessionID']
  end

  private
  def logout()
    response = @client["api/v1/logout"].get(
      :header => @header
    )
    
    @header[:'EnterpriseManager.APISessionID'] = ""
  end

  private
  def get_servers()
    response = @client["api/v1/servers"].get(
      :header => @header
    )
    return response.body.serverList
  end

  private
  def get_server_details(server)
    response = @client["api/v1/servers/#{server}"].get(
      :header => @header
    )
    return response.body.server_details
  end

  private
  def get_tasks(server)
    response = @client["api/v1/servers/#{server}/tasks"].get(
      :header => @header
    )
    return response.body.taskList
  end

  private
  def get_task_details(server, task)
    response = @client["api/v1/servers/#{server}/tasks/#{task}"].get(
      :header => @header
    )
    return response.body
  end

end
